## 前馈神经网络
尽管单个神经元比线性感知机更加强大，但表达能力还远不足以解决复杂的学习问题，这是我们大脑由不止一个神经元组成的原因。例如，单个神经元不可能区分开手写的数字。为了处理更加负载的任务，我们必须进一步优化机器学习模型。

大脑中的神经元是分层构成的。事实上，大脑皮质(负载人类大部分智商的结构)由6层组成 $^7$ 。信息流从一层到另一层直到感觉输入被转化为概念性理解。例如，视觉皮质末端层从眼睛接收原始的视觉数据，经每层处理后传递到下一层，直到第六层我们推断出正在看到的是只猫，或者汽水罐，抑或是架飞机。图1-9是这些层更简化的版本。

![A simple example of a feed-forward neural network with three layers (input, one hidden, and output) and three neurons per layer](https://github.com/lucasbyAI/Fundamental_of_Deep_Learning_ZH/blob/master/images_folder/Fig1-9.png?raw=true)

图1-9 A simple example of a feed-forward neural network with three layers (input, one hidden, and output) and three neurons per layer

借助这些概念，我们能够构建人工神经网络。当我们开始把神经元与其他神经元、输入数据和对学习问题相当于网络答复的输出节点连接起来，就实现了神经网络。图1-9展示了一个人工神经网络的简单例子，类似于McCulloch 和Pitt 1943年工作里的架构。网络的最底层吸收输入数据，神经元最顶层(输入节点)计算最终答案。神经元的中间层称为*隐藏层*，我们令 $k^{th}$ 层的 $i^{th}$ 个神经元与 $k+1^{st}$ 层的 $j^{th}$ 个神经元之间连接的加权为 $w_{i,j}^{(k)}$。这些加权组成了参数向量 $\theta$，如前所述，神经网络解决问题的能力取决于找到最优的值填入 $\theta$。

我们注意到在这个例子中，连接仅仅从较低层横贯到较高层，同层之间的神经元之间没有连接，没有从较高层向较低层传输数据的连接。这样的神经网络叫做*前馈*网络，由于易于分析，所以从讨论前馈网络开启我们的学习之旅。我们在第二章给出前馈网络分析(即为加权选择最优值的过程)。更多复杂的连接方式将在后续章节阐述。

在最后部分，我们将讨论在前馈神经网络中使用的层的主要类型。但是在讲解之前，必须牢记一些重要内容：

1. 正如我们提到的，夹在神经元第一层(输入层)和最后一层(输出层)之间的神经元层叫做隐藏层。当神经网络试图解决问题时，隐藏层就会产生最最奇妙的反应。(就像在手写数字例子中)，以前我们需要花费大量的精力识别有用的特征，然而现在隐藏层可以自动为我们识别有用特征。通常，看一下隐藏层的活动，它能够告诉你很多从数据中提取出来已经自动学习到的特征信息。

2. 尽管本例中各层有相同数量的神经元，但这既不是必须的也并不建议这么做。隐藏层往往比输入层神经元更少，以此来强迫网络学习到原始输入的压缩表示。例如，当眼睛从周围环境获取到原始像素值时，大脑把它们看做边和轮廓。这是因为大脑中生物神经元的隐藏层强迫我们对觉察到的每件事物提出更好的表示。

3. 不要求每个神经元的输出连接到下一层所有神经元的输入。事实上，选择哪个神经元连接到下一层哪些其他神经元是一项源自经验的艺术。当我们解决各种神经网络案例时再深度讨论这个问题。

4. 输入和输出都被向量化表示。例如，你可以想象一个神经网络，图像中单个像素RGB值作为输入被表示为向量(参考图1-3)。最后一层有两个神经元对应问题的答案：如果图像包含一条狗，输出[1,0]；如果图像包含一只猫，输出[0,1]；如果图像两者都包含，输出[1,1]；如果图像两者都不包含，输出[0,0]。

同样，我们观察到，与神经元的公式重写类似，可以在数学上把神经网络表示为一系列向量和矩阵运算。设网络 $i^{th}$ 层的输入为向量 $x=[x_1 \ x_2 \ ... \ x_n]$。输入通过神经元生成向量 $y=[y_1 \ y_2 \ ... \ y_m]$。如果我们构建大小为 $n×m$ 的加权矩阵 $W$ 和大小为 $m$ 的偏置向量，这个过程可以表示为简单的矩阵乘法。在矩阵中，每列对应一个神经元，列的 $j^{th}$ 元素对应于来自输入 $j^{th}$ 元素的连接加权。换句话说， $y=f(W^Tx+b)$，其中变换函数应用在向量上。当我们开始用软件实现这些网络时，这种公式重写将会变得愈加重要。

> 7 Mountcastle, Vernon B. “Modality and topographic properties of single neurons of cat’s somatic sensory cortex.”
Journal of Neurophysiology 20.4 (1957): 408-434.
