## 创建和操作TensorFlow变量
当我们用TensorFlow创建深度学习模型时，用变量表示模型中的参数。TensorFlow变量是内存中包含张量的缓冲区。张量仅当图执行时才实例化，而且过后立刻清除。变量与张量不一样，它历经多次图执行仍然存活。因此，TensorFlow变量有下述三点特性：

- 在图第一次使用之前，变量必须明确地实例化。
- 在每次迭代之后可以用梯度法更新变量，因为我们寻找模型的最优参数设置。
- 我们可以把变量中的值保存在硬盘上，后面使用的时候可以恢复。

这三点特性使得TensorFlow在创建机器学习模型时尤为有效。

创建变量很简单，TensorFlow提供机制允许我们用几种方法初始化变量。让我们以初始化描述前馈神经网络两层之间连接权重的变量开始：

```
weights = tf.Variable(tf.random_normal([300, 200], stddev=0.5), name="weights")
```

此处我们传了两个参数给tf.Variable $^6$。第一个是tf.random_normal $^7$，它是生成用标准差为0.5的正态分布初始化的张量的运算。我们指定这个张量的大小为300×200，表明权重连接300个神经元的层到200个神经元的层。我们也传了一个名字用于调用tf.Variable。这个名字是允许我们指定计算图中适当节点的唯一标识符。在这种情况下，权重是可训练的，或者说，我们将要对权重自动计算和使用梯度。如果权重不需要可训练，当我们调用tf.Variable时传一个可选标识：

```
weights = tf.Variable(tf.random_normal([300, 200], stddev=0.5), name="weights", trainable=False)
```

除了用tf.random_normal，还有其他几种方法初始化TensorFlow变量：

```
# Common tensors from the TensorFlow API docs
tf.zeros(shape, dtype=tf.float32, name=None)
tf.ones(shape, dtype=tf.float32, name=None)
tf.random_normal(shape, mean=0.0, stddev=1.0,
dtype=tf.float32, seed=None,
name=None)
tf.truncated_normal(shape, mean=0.0, stddev=1.0,
dtype=tf.float32, seed=None,
name=None)
tf.random_uniform(shape, minval=0, maxval=None,
dtype=tf.float32, seed=None,
name=None)
```

当我们调用tf.Variable时，三步运算加到了计算图中：
- 初始化变量时生成张量的运算
- tf.assign运算，负责在变量使用之前用初始化的张量填充值；
- 变量运算，保持变量当前值

![Three operations are added to the graph when instantiating a TensorFlow variable. In this example, we instantiate the variable weights using a random normal initializer.](https://github.com/lucasbyAI/Fundamental_of_Deep_Learning_ZH/blob/master/images_folder/Fig3-1.png?raw=true)

图3-1 Three operations are added to the graph when instantiating a TensorFlow variable. In this example, we instantiate the variable weights using a random normal initializer.

正如我们在前面三步运算中提到的，在使用任何TensorFlow变量之前，tf.assign $^8$ 运算必须执行使得变量被期望值适当初始化。执行tf.initialize_all_variables() $^9$ 触发图中所有的tf.assign运算，实现初始化。用tf.initialize_all_variables(var1,var2,...) $^{10}$ 我们也可以选择性地初始化计算图中的某些变量。在讨论TensorFlow会话时我们将更详细地阐述它。

> 6 https://www.tensorflow.org/api_docs/python/tf/Variable

> 7 https://www.tensorflow.org/api_docs/python/tf/random_normal

> 8 https://www.tensorflow.org/api_docs/python/tf/assign

> 9 http://bit.ly/2rtqoIA

> 10 https://www.tensorflow.org/api_docs/python/tf/initialize_variables
